#################################################################
### * * * Ćwiczenia 4 i 5 - Układy równań różniczkowych * * * ###
#################################################################

#Rozwiązania numeryczne równań rożniczkowych z wykorzystaniem pakietu deSolve
library(deSolve)


###############################################
### * * * Model Lotki-Volterry        * * * ###
###############################################

#Model Lotki-Volterry
LotVolt <- function (Time, State, Pars) {
  with(as.list(c(State, Pars)), {
    dV = r*V-a*V*P          # zmiany liczebnosci ofiar
    dP = -s*P+a*b*V*P       # zmiany liczebnosci drapieznikow
    return(list(c(dV, dP))) # zwracamy wyniki do ramki danych
  })
}

#Deklarujemy parametry
Pars <- c(r = 3.0, a =0.1, s = .5, b = .5)
State <- c(V = 10, P = 10)
Time <- seq(0, 30, by = 0.2)

#Zapisz rozwiazania ukladu w ramce danych:
wyniki <- as.data.frame(ode(func = LotVolt, y = State, parms = Pars, times = Time), method = rk4)

#Wykres rozwiazan:
matplot(wyniki[,-1], type = "l", xlab = "Czas", ylab = "Populacja ")
legend("topright", c("Ofiary", "Drapiezniki"), lty = c(1,2), col = c(1,2), box.lwd = 0)

#Trajektoria rozwiazan ukladu:
plot(wyniki$P,wyniki$V, type = "p", xlab = "Drapieznik (P)", ylab = "Ofiara (V)",pch=19,col="red")

############
# Izokliny #
############
options(device="RStudioGD")
#Parametry
r = 2.0; a =0.1; s = 0.5; b = 0.5

# Dodawanie linii do wykresu, funkcja abline(), argumenty:
# a - wyraz wolny funkcji
# b - wspolczynnik kierunkowy prostej
# h - wartosc y dla poziomej linii
# v - wartosc x dla pionowej linii

#Izoklina pozioma
V=s/(a*b)
abline(h=V) #Dodaj izokline dla ofiar na wykres

#Izoklina pionowa
P=r/a       #Dodaj izokline dla drapieznikow na wykres
abline(v=P) #Dodaj izokline dla drapieznikow na wykres

# Stosunek populacji ofiar do drapieżników
Ratio=V/P
Ratio

###########################
# * * * Zadanie 4.1 * * * #
###########################

# Jako wyjsciowe parametry przyjmij: r = 1.5; a =0.2; s = 0.5; b = 0.5
# Liczebnosci poczatkowe: V=10, P=10
# Do kazdego z podpunktow:
# - zapisz dwa wykresy: rozwiazania ukladu oraz trajektoria rozwiazan razem z izoklinami
# - wskaż punkt rownowagi (liczebnosci)

# Zwieksz lub zmniejsz dwukrotnie wspolczynniki modelu tak aby uwzglednic nastepujace sytuacje:
# A. Populacja ofiar ma bardzo szybkie tempo wzrostu, udzial ofiar upolowanych przez drapiezniki jest niewielki
# B. Populacja ofiar ma bardzo wolne tempo wzrostu, udzial ofiar upolowanych przez drapiezniki jest duzy
# C. Tylko niewielka czesc upolowanej biomasy jest wykorzystywana na rozrod drapieznikow, dodatkowo charakteryzuje je wysoka smiertelnosc
# D. Duza czesc upolowanej biomasy jest wykorzystywana na rozrod drapieznikow, dodatkowo charakteryzuje je niska smiertelnosc


###################################################
### * * * Model epidemiologiczny - SIR    * * * ###
###################################################

# Wprowadzenie: Zespol niewydolnosci oddechowej u swin (PRRS)
# Gospodarz posiadajacy stado 1000 sztuk zauwazyl jedno chore zwierze
# W kolejnych tygodniach zapisywal ilosc chorych osobnikow co 4 dni
# Po dwoch tygodniach chorowala polowa stada
# Po dwoch miesiacach epidemia wygasla
# Sredni czas trwania infekcji zostal oszacowany na 8.2 dnia

# Wektor czasu trwania epidemii - co 4 dni
czas<-seq(0,60,4)
czas

# Proporcja chorych zwierząt
chore<-c(0.03,0.08,0.23,0.4,0.57,0.44,0.31,0.21,0.12,0.04,0.05,0.03,0.02,0.02,0.01,0.02)
plot(czas,chore,pch=19,col="blue")

# Sredni czas trwania infekcji zostal oszacowany na 8.2 dnia więc a=1/średni czas trwania infekcji
a = 1/8.2
a

# Tempo infekcji zostalo oszacowane na 0.83 wiec
r = 0.83

# Funkcja: Model SIR
SIR.mod <- function (Time, State, Pars) {
  # Zmieniamy nazwy zmiennych i parametrow
  S<- State[1]
  I<- State[2]
  R<- State[3]
  N<- S+I+R
  r <- Pars[1]
  a <- Pars[2]
  
  # Uklad rownan
  dS<- - r*S*I
  dI <- r*S*I-a*I
  dR<- a*I
  
  # Zwraca rozwiazanie do listy
  list(c(dS,dI,dR))
}

# Stworzyliśmy nową funckję - sprawdzamy:
str(SIR.mod)

# Deklarujemy wartosci dla parametrow i umieszczamy w wektorze
r <- 0.83            # wspolczynnik infekcji
a <- 1.0/8.2         # wspolczynnik zdrowienia
SIR.par <- c(r,a)
SIR.par

# Warunki poczatkowe, kolejno frakcje S0 (podatne), I0 (chore) oraz R0 (odporne)
# Zakładamy, że jedno zwierzę na 1000 choruje
SIR.init <- c(0.999, 0.001, 0.000)

# Czas do obliczen
SIR.t <- seq(0,60, by=0.1)
SIR.t

# Numeryczne rozwiązanie równania za pomocą funkcji ode - metoda Runge - Kutty 4 rzędu
SIR.sol<- as.data.frame(ode(y = SIR.init,times = SIR.t,func = SIR.mod,parms = SIR.par,method = rk4))

# Sprawdzamy rozwiazania
head(SIR.sol)
names(SIR.sol)[2:4]<-c("S","I","R")     # zmiana nazw kolumn
SIR.sol$N<-apply(SIR.sol[,2:4],1,sum)

# Tworzymy wykres dla rozwiązan
# Podpis osi y
ylabel<-"Frakcja grup S, I oraz R" ; xlabel<-"Czas (dni)"

# Tytuł wykresu
ttl<-"Model SIR"

# Wykres
par(lwd=1)
with(SIR.sol,
     {plot(time,S,type="l",col="green",xlab=xlabel,ylim=c(0,1),ylab=ylabel,lwd=2,main=ttl)
       lines(time,I,col="red",lwd=2) ; lines(time,R,lwd=2)})

# Aby zmienić położenie legendy względem osi zmień liczy 40 (oś X) oraz 0.8 (oś Y)
legend(40,0.8,c("Podatne (S)","Chore (I)","Odporne (R)"),
       col=c("green","red","black"),lty=1,lwd=2)

# Badamy znak pochodnej
S = 0.1
I = 0.6
R = 0.3
dS<- - r*S*I
dI <- r*S*I-a*I
dR<- a*I

# PYTANIA:
# A. Oszacuj R0 - wspolczynnik reprodukcji infekcji R0. Czy epidemia wybuchnie?
# Szacujemy R0
R0 = (r*0.999)/a

# Epidemia wybuchnie (frakcja zainfekowanych bedzie rosla) jezeli frakcja podatnych S0 będzie wieksza niż 1 / R0
1 / R0

# B. Jaka jest maksymalna frakcja chorych jaka bedzie obserwowana w trakcie epidemii?
SIR.sol[SIR.sol$I==max(SIR.sol$I),]

# Sprawdź: maksymalna frakcja chorych powinna się pojawić kiedy S = a/r
# jest to równocześnie współczynnik usuwania zarażonych osobników (ang. relative removal rate)
Podatne_S <- a/r
Podatne_S

# C. Kiedy frakcja chorych spadnie ponizej 5%?
# Liczba 12 wynika z punktu 1, pozwala ominąć pierwsze 12 dni kiedy liczba chorych rosła
min(SIR.sol[SIR.sol$time > 12 & SIR.sol$I < 0.05,]$time)

# D. Jaka część populacji przejdzie infekcje i jaka bedzie frakcja zdrowych i podatnych na koniec epidemii?
# Sprawdzamy maksymalną frakcje podatnych
max(SIR.sol$R)
1-max(SIR.sol$R)

############################
# * * * Zadanie 4.2 * * * #
###########################
# Utworz wykres rozwiazan oraz odpowiedz na pytania 1-4 (wyżej) dla nastepujacych sytuacji

# 1A. Polowa stada jest odporna w wyniku przebycia infekcji wczesniej
# Wspolczynniki pozostaja na stalym poziomie
# Jak zmienia sie warunki poczatkowe?
# Jaka powinna byc frakcja odpornych aby uniknac kolejnej epidemii?

# Deklarujemy wartosci dla parametrow i umieszczamy w wektorze

# Warunki poczatkowe, kolejno frakcje S0 (podatne), I0 (chore) oraz R0 (odporne)

# Czas do obliczen

# Model

#########################################
# PYTANIA:

# A. Oszacuj R0 - wspolczynnik reprodukcji infekcji R0. Czy epidemia wybuchnie?

# B. Jaka jest maksymalna frakcja chorych jaka bedzie obserwowana w trakcie epidemii?

# Sprawdź: maksymalna frakcja chorych powinna się pojawić kiedy S = a/r
# jest to równocześnie współczynnik usuwania zarażonych osobników (ang. relative removal rate)

# C. Kiedy frakcja chorych spadnie ponizej 5%?
# Liczba 12 wynika z punktu 1, pozwala ominąć pierwsze 12 dni kiedy liczba chorych rosła

# D. Jaka część populacji przejdzie infekcje i jaka bedzie frakcja zdrowych i podatnych na koniec epidemii?
# Sprawdzamy maksymalną frakcje podatnych

#########################################
# 1B. Zakladamy że w stadzie pojawia sie 2-razy bardziej zjadliwy szczep wirusa.
# Wspolczynnik r podwaja się do 1.66
# Warunki poczatkowe pozostaja jak w przykladowej analizie

# 1C. Zakladamy ze w wyniku podjetego leczenia infekcja trwa tylko dwa dni
# Wspolczynnik r = 0.83 bez zmian, wspolczynnik a zmienia sie
# Warunki poczatkowe pozostaja jak w przykladowej analizie

###########################
# * * * Zadanie 4.3 * * * #
###########################
# Jak przebiega epidemia sezonowej grypy?
# Zaczynamy od jednej chorej osoby, wprowadzonej do populacji, w której nikt nie ma odporności
# Tempo infekcji r wynosi 0.29
# Infekcja trwa przeciętnie tydzień, oblicz a
# Zakładamy, że nabywamy odporności na dany szczep po przechorowaniu
# Odpowiedz na pytania A - D

# *** Dla chętnych ***
# Stwórz model SIS dla danych z zadania 2 zakładając brak odporności
# i pokaż przykładowe rozwiązania na wykresie

###########################
# * * * Zadanie 4.4 * * * #
###########################
# Wejdz na stronę: https://alhill.shinyapps.io/COVID19seir/
# i krótko opisz jak zmienia się przebieg epidemii i Ro
# Pomiędzy symulacjami zresetuj parametry!
# 1. Zwiększ okres inkubacji do 15 dni.
# 2. Wydłuż czas trwania infekcji (mild) do 8 dni
# 3. Zmień tempo transmisji dla łagodnych infekcji do 1.5
# 4. Włącz:
# A. infekcje bezobjawowe (allow asymptomatic infections)
# B. możliwość transmisji choroby przed wystapieniem objawów (allow pre-symptomatic transmission)


